==> 3.13.1.zip: libjf/libjf-config-commands/src/main/java/io/gitlab/jfronny/libjf/config/impl/commands/JfConfigCommand.java <==
package io.gitlab.jfronny.libjf.config.impl.commands;

import com.mojang.brigadier.Command;
import com.mojang.brigadier.arguments.*;
import com.mojang.brigadier.builder.LiteralArgumentBuilder;
import com.mojang.brigadier.exceptions.CommandSyntaxException;
import com.mojang.brigadier.exceptions.DynamicCommandExceptionType;
import io.gitlab.jfronny.commons.throwable.ThrowingRunnable;
import io.gitlab.jfronny.commons.throwable.ThrowingSupplier;
import io.gitlab.jfronny.libjf.LibJf;
import io.gitlab.jfronny.libjf.config.api.v2.*;
import io.gitlab.jfronny.libjf.config.api.v2.type.Type;
import net.fabricmc.api.ModInitializer;
import net.fabricmc.fabric.api.command.v2.CommandRegistrationCallback;
import net.minecraft.server.command.ServerCommandSource;
import net.minecraft.text.MutableText;
import net.minecraft.text.Text;

import java.util.function.Consumer;
import java.util.function.Function;

import static net.minecraft.server.command.CommandManager.argument;
import static net.minecraft.server.command.CommandManager.literal;

public class JfConfigCommand implements ModInitializer {
    private static final String MOD_ID = "libjf-config-commands";

    private MutableText text(String text) {
        return Text.literal("[" + MOD_ID + "] " + text);
    }

    @Override
    public void onInitialize() {
        CommandRegistrationCallback.EVENT.register((dispatcher, registryAccess, environment) -> {
            LiteralArgumentBuilder<ServerCommandSource> c_libjf = literal(LibJf.MOD_ID);
            LiteralArgumentBuilder<ServerCommandSource> c_config = literal("config")
                    .requires((serverCommandSource) -> serverCommandSource.hasPermissionLevel(4))
                    .executes(context -> {
                        MutableText text = text("Loaded configs for:");
                        ConfigHolder.getInstance().getRegistered().forEach((s, config) -> text.append("\n- " + s));
                        context.getSource().sendFeedback(() -> text, false);
                        return Command.SINGLE_SUCCESS;
                    });
            LiteralArgumentBuilder<ServerCommandSource> c_reload = literal("reload").executes(context -> {
                ConfigHolder.getInstance().getRegistered().forEach((mod, config) -> config.load());
                context.getSource().sendFeedback(() -> text("Reloaded configs"), true);
                return Command.SINGLE_SUCCESS;
            });
            LiteralArgumentBuilder<ServerCommandSource> c_reset = literal("reset").executes(context -> {
                context.getSource().sendError(text("Please specify a config to reset"));
                return Command.SINGLE_SUCCESS;
            });
            ConfigHolder.getInstance().getRegistered().forEach((id, config) -> {
                c_reload.then(literal(id).executes(context -> {
                    config.load();
                    context.getSource().sendFeedback(() -> text("Reloaded config for " + id), true);
                    return Command.SINGLE_SUCCESS;
                }));
                registerEntries(config, id, c_config, c_reset, cns -> {
                    LiteralArgumentBuilder<ServerCommandSource> c_instance = literal(id);
                    cns.accept(c_instance);
                    return c_instance;
                });
            });
            dispatcher.register(c_libjf.then(c_config.then(c_reload).then(c_reset)));
        });
    }

    private void registerEntries(ConfigCategory config, String subpath, LiteralArgumentBuilder<ServerCommandSource> c_config, LiteralArgumentBuilder<ServerCommandSource> c_reset, Function<Consumer<LiteralArgumentBuilder<ServerCommandSource>>, LiteralArgumentBuilder<ServerCommandSource>> pathGen) {
        c_config.then(pathGen.apply(cns -> {
            cns.executes(context -> {
                context.getSource().sendFeedback(() -> text(subpath + " is a category"), false);
                return Command.SINGLE_SUCCESS;
            });
            for (EntryInfo<?> entry : config.getEntries()) {
                registerEntry(config, subpath, cns, entry);
            }
        }));
        c_reset.then(pathGen.apply(cns -> {
            cns.executes(context -> {
                config.reset();
                context.getSource().sendFeedback(() -> text("Reset config for " + subpath), true);
                return Command.SINGLE_SUCCESS;
            });
            config.getPresets().forEach((id2, preset) -> {
                cns.then(literal(id2).executes(context -> {
                    preset.run();
                    context.getSource().sendFeedback(() -> text("Loaded preset " + id2 + " for " + subpath), true);
                    return Command.SINGLE_SUCCESS;
                }));
            });
        }));
        config.getCategories().forEach((id2, cfg) -> {
            registerEntries(cfg, cfg.getCategoryPath(), c_config, c_reset, cns -> {
                return pathGen.apply(cns1 -> {
                    LiteralArgumentBuilder<ServerCommandSource> c_instance2 = literal(id2);
                    cns.accept(c_instance2);
                    cns1.then(c_instance2);
                });
            });
        });
    }

    private final DynamicCommandExceptionType eType = new DynamicCommandExceptionType(o -> {
        if (o instanceof Throwable throwable) {
            return Text.literal("Could not execute command: " + throwable.getMessage());
        } else return Text.literal("Could not execute command");
    });

    private <T> void registerEntry(ConfigCategory config, String subpath, LiteralArgumentBuilder<ServerCommandSource> cns, EntryInfo<T> entry) {
        LiteralArgumentBuilder<ServerCommandSource> c_entry = literal(entry.getName()).executes(context -> {
            String msg = "The value of " + subpath + "." + entry.getName() + " is " + tryRun(entry::getValue);
            context.getSource().sendFeedback(() -> text(msg), false);
            return Command.SINGLE_SUCCESS;
        });
        ArgumentType<?> type = getType(entry);
        if (type != null) {
            c_entry.then(argument("value", type).executes(context -> {
                @SuppressWarnings("unchecked") T value = context.getArgument("value", (Class<T>) entry.getValueType().asClass());
                tryRun(() -> {
                    entry.setValue(value);
                    config.getRoot().write();
                });
                context.getSource().sendFeedback(() -> text("Set " + subpath + "." + entry.getName() + " to " + value), true);
                return Command.SINGLE_SUCCESS;
            }));
        }
        else if (entry.getValueType().isEnum()) {
            for (T enumConstant : ((Type.TEnum<T>)entry.getValueType()).options()) {
                c_entry.then(literal(enumConstant.toString()).executes(context -> {
                    tryRun(() -> {
                        entry.setValue(enumConstant);
                        config.getRoot().write();
                    });
                    context.getSource().sendFeedback(() -> text("Set " + subpath + "." + entry.getName() + " to " + enumConstant), true);
                    return Command.SINGLE_SUCCESS;
                }));
            }
        }
        cns.then(c_entry);
    }

    private <T> ArgumentType<?> getType(EntryInfo<T> info) {
        Type type = info.getValueType();
        if (type.isInt()) return IntegerArgumentType.integer((int) info.getMinValue(), (int) info.getMaxValue());
        else if (type.isLong()) return LongArgumentType.longArg((long) info.getMinValue(), (long) info.getMaxValue());
        else if (type.isFloat()) return FloatArgumentType.floatArg((float) info.getMinValue(), (float) info.getMaxValue());
        else if (type.isDouble()) return DoubleArgumentType.doubleArg(info.getMinValue(), info.getMaxValue());
        else if (type.isString()) return StringArgumentType.greedyString();
        else if (type.isBool()) return BoolArgumentType.bool();
        else return null;
    }

    private <T> T tryRun(ThrowingSupplier<T, ?> supplier) throws CommandSyntaxException {
        return supplier.orThrow(eType::create).get();
    }

    private void tryRun(ThrowingRunnable<?> supplier) throws CommandSyntaxException {
        supplier.orThrow(eType::create).run();
    }
}

==> 3.15.7.zip: libjf/libjf-config-commands/src/main/java/io/gitlab/jfronny/libjf/config/impl/commands/JfConfigCommand.java <==
package io.gitlab.jfronny.libjf.config.impl.commands;

import com.mojang.brigadier.Command;
import com.mojang.brigadier.arguments.*;
import com.mojang.brigadier.builder.LiteralArgumentBuilder;
import com.mojang.brigadier.exceptions.CommandSyntaxException;
import com.mojang.brigadier.exceptions.DynamicCommandExceptionType;
import io.gitlab.jfronny.commons.throwable.ThrowingRunnable;
import io.gitlab.jfronny.commons.throwable.ThrowingSupplier;
import io.gitlab.jfronny.libjf.LibJf;
import io.gitlab.jfronny.libjf.config.api.v2.*;
import io.gitlab.jfronny.libjf.config.api.v2.type.Type;
import io.gitlab.jfronny.libjf.config.impl.ConfigCore;
import net.fabricmc.api.ModInitializer;
import net.fabricmc.fabric.api.command.v2.CommandRegistrationCallback;
import net.minecraft.server.command.ServerCommandSource;
import net.minecraft.text.MutableText;
import net.minecraft.text.Text;
import net.minecraft.util.Language;

import java.util.function.Consumer;
import java.util.function.Function;

import static net.minecraft.server.command.CommandManager.argument;
import static net.minecraft.server.command.CommandManager.literal;

public class JfConfigCommand implements ModInitializer {
    private static final String MOD_ID = "libjf-config-commands";

    private MutableText text(String text) {
        return Text.literal("[" + MOD_ID + "] " + text);
    }

    @Override
    public void onInitialize() {
        CommandRegistrationCallback.EVENT.register((dispatcher, registryAccess, environment) -> {
            LiteralArgumentBuilder<ServerCommandSource> c_libjf = literal(LibJf.MOD_ID);
            LiteralArgumentBuilder<ServerCommandSource> c_config = literal("config")
                    .requires((serverCommandSource) -> serverCommandSource.hasPermissionLevel(4))
                    .executes(context -> {
                        MutableText text = text("Loaded configs for:");
                        ConfigHolder.getInstance().getRegistered().forEach((s, config) -> text.append("\n- " + s));
                        context.getSource().sendFeedback(() -> text, false);
                        return Command.SINGLE_SUCCESS;
                    });
            LiteralArgumentBuilder<ServerCommandSource> c_reload = literal("reload").executes(context -> {
                ConfigHolder.getInstance().getRegistered().forEach((mod, config) -> config.load());
                context.getSource().sendFeedback(() -> text("Reloaded configs"), true);
                return Command.SINGLE_SUCCESS;
            });
            LiteralArgumentBuilder<ServerCommandSource> c_reset = literal("reset").executes(context -> {
                context.getSource().sendError(text("Please specify a config to reset"));
                return Command.SINGLE_SUCCESS;
            });
            ConfigHolder.getInstance().getRegistered().forEach((id, config) -> {
                c_reload.then(literal(id).executes(context -> {
                    config.load();
                    context.getSource().sendFeedback(() -> text("Reloaded config for " + id), true);
                    return Command.SINGLE_SUCCESS;
                }));
                registerEntries(config, id, c_config, c_reset, cns -> {
                    LiteralArgumentBuilder<ServerCommandSource> c_instance = literal(id);
                    cns.accept(c_instance);
                    return c_instance;
                });
            });
            dispatcher.register(c_libjf.then(c_config.then(c_reload).then(c_reset)));
        });
    }

    private void registerEntries(ConfigCategory config, String subpath, LiteralArgumentBuilder<ServerCommandSource> c_config, LiteralArgumentBuilder<ServerCommandSource> c_reset, Function<Consumer<LiteralArgumentBuilder<ServerCommandSource>>, LiteralArgumentBuilder<ServerCommandSource>> pathGen) {
        c_config.then(pathGen.apply(cns -> {
            cns.executes(context -> {
                context.getSource().sendFeedback(() -> text(subpath + " is a category"), false);
                return Command.SINGLE_SUCCESS;
            });
            for (EntryInfo<?> entry : config.getEntries()) {
                registerEntry(config, subpath, cns, entry);
            }
        }));
        c_reset.then(pathGen.apply(cns -> {
            cns.executes(context -> {
                config.reset();
                context.getSource().sendFeedback(() -> text("Reset config for " + subpath), true);
                return Command.SINGLE_SUCCESS;
            });
            config.getPresets().forEach((id2, preset) -> {
                cns.then(literal(id2).executes(context -> {
                    preset.run();
                    context.getSource().sendFeedback(() -> text("Loaded preset " + id2 + " for " + subpath), true);
                    return Command.SINGLE_SUCCESS;
                }));
            });
        }));
        config.getCategories().forEach((id2, cfg) -> {
            registerEntries(cfg, cfg.getCategoryPath(), c_config, c_reset, cns -> {
                return pathGen.apply(cns1 -> {
                    LiteralArgumentBuilder<ServerCommandSource> c_instance2 = literal(id2);
                    cns.accept(c_instance2);
                    cns1.then(c_instance2);
                });
            });
        });
    }

    private final DynamicCommandExceptionType eType = new DynamicCommandExceptionType(o -> {
        if (o instanceof Throwable throwable) {
            return Text.literal("Could not execute command: " + throwable.getMessage());
        } else return Text.literal("Could not execute command");
    });

    private <T> void registerEntry(ConfigCategory config, String subpath, LiteralArgumentBuilder<ServerCommandSource> cns, EntryInfo<T> entry) {
        if (!entry.supportsRepresentation()) return;
        LiteralArgumentBuilder<ServerCommandSource> c_entry = literal(entry.getName()).executes(context -> {
            String msg = "The value of " + subpath + "." + entry.getName() + " is ";
            Text visualized = visualizeOption(config, entry, tryRun(entry::getValue));
            context.getSource().sendFeedback(() -> text(msg).append(visualized), false);
            return Command.SINGLE_SUCCESS;
        });
        ArgumentType<?> type = getType(entry);
        if (type != null) {
            c_entry.then(argument("value", type).executes(context -> {
                @SuppressWarnings("unchecked") T value = context.getArgument("value", (Class<T>) entry.getValueType().asClass());
                tryRun(() -> {
                    entry.setValue(value);
                    config.getRoot().write();
                });
                context.getSource().sendFeedback(() -> text("Set " + subpath + "." + entry.getName() + " to ")
                        .append(visualizeOption(config, entry, value)), true);
                return Command.SINGLE_SUCCESS;
            }));
        }
        else if (entry.getValueType().isEnum()) {
            for (T enumConstant : ((Type.TEnum<T>)entry.getValueType()).options()) {
                c_entry.then(literal(enumConstant.toString()).executes(context -> {
                    tryRun(() -> {
                        entry.setValue(enumConstant);
                        config.getRoot().write();
                    });
                    context.getSource().sendFeedback(() -> text("Set " + subpath + "." + entry.getName() + " to " + enumConstant)
                            .append(visualizeOption(config, entry, enumConstant)), true);
                    return Command.SINGLE_SUCCESS;
                }));
            }
        }
        cns.then(c_entry);
    }

    private <T> Text visualizeOption(ConfigCategory config, EntryInfo<T> entry, T value) {
        Language lang = Language.getInstance();
        String key = null;
        Type type = entry.getValueType();
        if (value == null) {
            key = ConfigCore.MOD_ID + ".null";
        } if (type.isEnum()) {
            key = type.asClass() == null
                    ? config.getTranslationPrefix() + entry.getName() + "." + value
                    : config.getTranslationPrefix() + "enum." + type.getName() + "." + value;
        } else if (type.isBool()) {
            String customKey = config.getTranslationPrefix() + value;
            key = lang.hasTranslation(customKey)
                    ? customKey
                    : ConfigCore.MOD_ID + "." + value;
        }
        if (key == null) return Text.literal(String.valueOf(value));
        else if (lang.hasTranslation(key)) return Text.translatableWithFallback(key, lang.get(key));
        else return Text.translatableWithFallback(key, String.valueOf(value));
    }

    private <T> ArgumentType<?> getType(EntryInfo<T> info) {
        Type type = info.getValueType();
        if (type.isInt()) return IntegerArgumentType.integer((int) info.getMinValue(), (int) info.getMaxValue());
        else if (type.isLong()) return LongArgumentType.longArg((long) info.getMinValue(), (long) info.getMaxValue());
        else if (type.isFloat()) return FloatArgumentType.floatArg((float) info.getMinValue(), (float) info.getMaxValue());
        else if (type.isDouble()) return DoubleArgumentType.doubleArg(info.getMinValue(), info.getMaxValue());
        else if (type.isString()) return StringArgumentType.greedyString();
        else if (type.isBool()) return BoolArgumentType.bool();
        else return null;
    }

    private <T> T tryRun(ThrowingSupplier<T, ?> supplier) throws CommandSyntaxException {
        return supplier.orThrow(eType::create).get();
    }

    private void tryRun(ThrowingRunnable<?> supplier) throws CommandSyntaxException {
        supplier.orThrow(eType::create).run();
    }
}
